﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Drawing.Imaging;
using System.IO;
using System.Runtime.InteropServices;
using System.Threading;
using System.Threading.Tasks;
using System.Windows.Forms;
using ff14bot;
using ff14bot.Helpers;
using ff14bot.Interfaces;
using ff14bot.Objects;

namespace EnemySpellLogger
{
    public class EnemySpellLogger : IBotPlugin
    {
        private object locker = new object();
        private const uint WM_KEYDOWN = 0x100;
        private const uint WM_KEYUP = 0x0101;
        private HashSet<uint> loggedSpells = new HashSet<uint>();
        private const string directory = "Enemy Spell Logger";
        private IntPtr windowHandle;
        private volatile bool enabled = true;
        private volatile bool uiHidden;
        private const int waitTimeBeforeScreenshot = 200;

        private void HideOrShowUI(bool hide)
        {
            lock (locker)
            {
                if (uiHidden && hide) { return; }

                PostMessage(windowHandle, WM_KEYDOWN, (IntPtr)(Keys.Scroll), IntPtr.Zero);
                PostMessage(windowHandle, WM_KEYUP, (IntPtr)(Keys.Scroll), IntPtr.Zero);

                uiHidden = hide;
            }
        }

        public string Author
        {
            get { return " Shifterino Pepperonino"; }
        }

        public string ButtonText
        {
            get { return "Settings"; }
        }

        public string Description
        {
            get { return "Logs enemy spell casts by saving a picture named as the spell's ID."; }
        }

        public string Name
        {
            get { return "Enemy Spell Logger"; }
        }

        public void OnButtonPress()
        {
            
        }

        public void OnDisabled()
        {
            
        }

        public void OnEnabled()
        {
            
        }

        public void OnInitialize()
        {
            windowHandle = Core.Memory.Process.MainWindowHandle;

            if (!Directory.Exists(directory))
            {
                try { Directory.CreateDirectory(directory); }
                catch
                {
                    Logging.Write("Could not create screenshot directory.");
                    enabled = false;
                }
            }
        }

        public void TakePicture(string name)
        {
            Thread.Sleep(waitTimeBeforeScreenshot);

            Bitmap image;

            HideOrShowUI(true);

            Thread.Sleep(200);

            try { image = PrintWindow(windowHandle); }
            catch (Exception e)
            {
                Logging.Write(string.Format("Could not save screenshot because: {0}", e));
                enabled = false; 
                return;
            }

            HideOrShowUI(false);

            var path = Path.Combine(directory, name) + ".png";
            try
            {
                using (var file = new FileStream(path, FileMode.Create)) { image.Save(file, ImageFormat.Png); }
            }
            catch (Exception e)
            {
                Logging.Write(string.Format("Could not save screenshot because: {0}", e));
                enabled = false; 
                return;
            }

            Logging.Write(string.Format("[Enemy Spell Logger] Screenshot captured for {0}.", name));
        }

        public void OnPulse()
        {
            if (!enabled) { return; }

            var target = Core.Me.CurrentTarget as BattleCharacter;
            if (target == null || !target.IsCasting || target.SpellCastInfo == null) { return; }

            var actionId = target.SpellCastInfo.ActionId;
            if (loggedSpells.Contains(actionId)) { return; }

            loggedSpells.Add(actionId);
            var name = string.Format("{0} - {1}", target.SpellCastInfo.Name, target.SpellCastInfo.ActionId);

            Task.Run(() => TakePicture(name));
        }

        public void OnShutdown()
        {
            
        }

        public Version Version
        {
            get { return new Version(0, 0); }
        }

        public bool WantButton
        {
            get { return false; }
        }

        public bool Equals(IBotPlugin other)
        {
            return other.Author == Author && other.Name == Name;
        }

        [DllImport("user32.dll")]
        public static extern bool GetWindowRect(IntPtr hWnd, out RECT lpRect);
        [DllImport("user32.dll")]
        public static extern bool PrintWindow(IntPtr hWnd, IntPtr hdcBlt, int nFlags);
        [DllImport("user32.dll")]
        public static extern IntPtr PostMessage(IntPtr hWnd, uint Msg, IntPtr wParam, IntPtr lParam);

        public static Bitmap PrintWindow(IntPtr hwnd)
        {
            RECT rc;
            GetWindowRect(hwnd, out rc);

            Bitmap bmp = new Bitmap(rc.Width, rc.Height, PixelFormat.Format16bppRgb555);
            Graphics gfxBmp = Graphics.FromImage(bmp);
            IntPtr hdcBitmap = gfxBmp.GetHdc();

            PrintWindow(hwnd, hdcBitmap, 0);

            gfxBmp.ReleaseHdc(hdcBitmap);
            gfxBmp.Dispose();

            return bmp;
        }
    }
}
